@@ -88,12 +88,10 @@ extern "C"
 
     void        AMF_CDECL_CALL amf_increase_timer_precision();
     void        AMF_CDECL_CALL amf_restore_timer_precision();
-#if defined(_WIN32)
+
     amf_handle  AMF_CDECL_CALL amf_load_library(const wchar_t* filename);
-#else
-    amf_handle  AMF_STD_CALL amf_load_library(const wchar_t* filename);
-    amf_handle  AMF_STD_CALL amf_load_library1(const wchar_t* filename, bool bGlobal);
-#endif
+    amf_handle  AMF_CDECL_CALL amf_load_library1(const wchar_t* filename, bool bGlobal);
+
     void*       AMF_CDECL_CALL amf_get_proc_address(amf_handle module, const char* procName);
     int         AMF_CDECL_CALL amf_free_library(amf_handle module);
 
@@ -316,7 +314,7 @@ namespace amf
         virtual bool RequestStop();
         virtual bool WaitForStop();
         virtual bool StopRequested();
-        virtual bool IsRunning();
+        virtual bool IsRunning() const;
 
         // this is executed in the thread and overloaded by implementor
         virtual void Run() = 0;
@@ -547,7 +545,7 @@ namespace amf
                 {
                     bStop = true;
                 }
-#if defined(__linux)
+#if defined(__linux) || defined(__APPLE__)
                 ///< HACK
                 ///< This amf_sleep(0) is required to emulate windows mutex behavior.
                 ///< In Windows release mutext causes some other waiting thread is receiving ownership of mutex.
