@@ -63,8 +63,8 @@ static const wchar_t* pCodec = AMFVideoEncoderVCE_AVC;
 //static const wchar_t *pCodec = AMFVideoEncoder_HEVC;
 static const bool isH264 = (amf_wstring(pCodec) == amf_wstring(AMFVideoEncoderVCE_AVC)) ? true : false;
 
-//static const amf_int picMgmtMode = (isH264) ? AMF_VIDEO_ENCODER_PICTURE_TRANSFER_MODE_OFF : AMF_VIDEO_ENCODER_HEVC_PICTURE_TRANSFER_MODE_OFF; // disable ref/recon pic transfer
-static const amf_int picMgmtMode = (isH264) ? AMF_VIDEO_ENCODER_PICTURE_TRANSFER_MODE_ON : AMF_VIDEO_ENCODER_HEVC_PICTURE_TRANSFER_MODE_ON;     // enable ref/recon pic transfer
+//static const amf_int64 picMgmtMode = (isH264) ? AMF_VIDEO_ENCODER_PICTURE_TRANSFER_MODE_OFF : AMF_VIDEO_ENCODER_HEVC_PICTURE_TRANSFER_MODE_OFF; // disable ref/recon pic transfer
+static const amf_int64 picMgmtMode = (isH264) ? amf_int64(AMF_VIDEO_ENCODER_PICTURE_TRANSFER_MODE_ON) : amf_int64(AMF_VIDEO_ENCODER_HEVC_PICTURE_TRANSFER_MODE_ON);     // enable ref/recon pic transfer
 
 static const bool injectReconDump = true; // inject the reconstructed picture dump from the encoder
 //static const bool injectReconDump = false; // inject the frame generated by this sample
@@ -222,8 +222,8 @@ int main(int argc, char* argv[])
 
     if(isH264)
     { 
-        res = encoder->SetProperty(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCONDING);
-        AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCONDING) failed");
+        res = encoder->SetProperty(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCODING);
+        AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCODING) failed");
 
         if(bMaximumSpeed)
         {
@@ -235,8 +235,6 @@ int main(int argc, char* argv[])
 
         res = encoder->SetProperty(AMF_VIDEO_ENCODER_TARGET_BITRATE, bitRateIn);
         AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_TARGET_BITRATE, %" LPRId64 L") failed", bitRateIn);
-        res = encoder->SetProperty(AMF_VIDEO_ENCODER_FRAMESIZE, ::AMFConstructSize(widthIn, heightIn));
-        AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_FRAMESIZE, %dx%d) failed", widthIn, heightIn);
         res = encoder->SetProperty(AMF_VIDEO_ENCODER_FRAMERATE, ::AMFConstructRate(frameRateIn, 1));
         AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_FRAMERATE, %dx%d) failed", frameRateIn, 1);
         res = encoder->SetProperty(AMF_VIDEO_ENCODER_PICTURE_TRANSFER_MODE, picMgmtMode);
@@ -254,8 +252,8 @@ int main(int argc, char* argv[])
     }
     else
     {
-        res = encoder->SetProperty(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCONDING);
-        AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCONDING)");
+        res = encoder->SetProperty(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCODING);
+        AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCODING)");
 
         if(bMaximumSpeed)
         {
@@ -265,8 +263,6 @@ int main(int argc, char* argv[])
 
         res = encoder->SetProperty(AMF_VIDEO_ENCODER_HEVC_TARGET_BITRATE, bitRateIn);
         AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_HEVC_TARGET_BITRATE, %" LPRId64 L") failed", bitRateIn);
-        res = encoder->SetProperty(AMF_VIDEO_ENCODER_HEVC_FRAMESIZE, ::AMFConstructSize(widthIn, heightIn));
-        AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_HEVC_FRAMESIZE, %dx%d) failed", widthIn, heightIn);
         res = encoder->SetProperty(AMF_VIDEO_ENCODER_HEVC_FRAMERATE, ::AMFConstructRate(frameRateIn, 1));
         AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_HEVC_FRAMERATE, %dx%d) failed", frameRateIn, 1);
         res = encoder->SetProperty(AMF_VIDEO_ENCODER_HEVC_PICTURE_TRANSFER_MODE, picMgmtMode);
