@@ -39,6 +39,7 @@
 #include "public/include/components/VideoEncoderVCE.h"
 #include "public/include/components/VideoEncoderHEVC.h"
 #include "public/include/components/VideoConverter.h"
+#include "public/include/components/HQScaler.h"
 
 
 std::wstring AddIndexToPath(const std::wstring& path, amf_int32 index)
@@ -305,6 +306,10 @@ AMF_RESULT ParamConverterFormat(const std::wstring& value, amf::AMFVariant& valu
         paramValue =  amf::AMF_SURFACE_YUY2;
     } else if(uppValue == L"P010" || uppValue == L"10") {
         paramValue =  amf::AMF_SURFACE_P010;
+    } else if (uppValue == L"P012" || uppValue == L"12") {
+        paramValue = amf::AMF_SURFACE_P012;
+    }else if (uppValue == L"P016" || uppValue == L"16") {
+        paramValue = amf::AMF_SURFACE_P016;
     } else if(uppValue == L"RGBAF16" || uppValue == L"RGBA_F16" || uppValue == L"11") {
         paramValue =  amf::AMF_SURFACE_RGBA_F16;
     } else if(uppValue == L"UYVY" || uppValue == L"12") {
@@ -593,3 +598,46 @@ const wchar_t *StreamCodecIDtoDecoderID(AMF_STREAM_CODEC_ID_ENUM eCodec)
     }
     return L"";
 }
+
+AMF_RESULT ParamConverterHQScalerAlgorithm(const std::wstring& value, amf::AMFVariant& valueOut)
+{
+    amf_int64    paramValue = -1;
+    std::wstring uppValue   = toUpper(value);
+    if ((uppValue == L"BILINEAR") || (uppValue == L"0"))
+    {
+        paramValue = AMF_HQ_SCALER_ALGORITHM_BILINEAR;
+    }
+    else if ((uppValue == L"BICUBIC") || (uppValue == L"1"))
+    {
+        paramValue = AMF_HQ_SCALER_ALGORITHM_BICUBIC;
+    }
+    else if ((uppValue == L"FSR") || (uppValue == L"2"))
+    {
+        paramValue = AMF_HQ_SCALER_ALGORITHM_FSR;
+    }
+
+    valueOut = amf_int64(paramValue);
+    return AMF_OK;
+}
+
+AMF_RESULT ParamConverterHighMotionQualityBoostMode(const std::wstring& value, amf::AMFVariant& valueOut)
+{
+    AMF_PA_HIGH_MOTION_QUALITY_BOOST_MODE_ENUM paramValue;
+
+    std::wstring uppValue = toUpper(value);
+    if (uppValue == L"NONE" || uppValue == L"0")
+    {
+        paramValue = AMF_PA_HIGH_MOTION_QUALITY_BOOST_MODE_NONE;
+    }
+    else if (uppValue == L"AUTO" || uppValue == L"1")
+    {
+        paramValue = AMF_PA_HIGH_MOTION_QUALITY_BOOST_MODE_AUTO;
+    }
+    else
+    {
+        LOG_ERROR(L"AMF_PA_HIGH_MOTION_QUALITY_BOOST_MODE_ENUM hasn't \"" << value << L"\" value.");
+        return AMF_INVALID_ARG;
+    }
+    valueOut = amf_int64(paramValue);
+    return AMF_OK;
+}
