@@ -1,4 +1,4 @@
-// 
+//
 // Notice Regarding Standards.  AMD does not provide a license or sublicense to
 // any Intellectual Property Rights relating to any standards, including but not
 // limited to any audio and/or video codec technologies such as MPEG-2, MPEG-4;
@@ -6,9 +6,9 @@
 // (collectively, the "Media Technologies"). For clarity, you will pay any
 // royalties due for such third party technologies, which may include the Media
 // Technologies that are owed as a result of AMD providing the Software to you.
-// 
-// MIT license 
-// 
+//
+// MIT license
+//
 // Copyright (c) 2018 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -49,7 +49,12 @@ const wchar_t* PlaybackPipelineBase::PARAM_NAME_DOTIMING = L"DOTIMING";
 const wchar_t* PlaybackPipelineBase::PARAM_NAME_LOWLATENCY = L"LOWLATENCY";
 const wchar_t* PlaybackPipelineBase::PARAM_NAME_FULLSCREEN = L"FULLSCREEN";
 const wchar_t* PlaybackPipelineBase::PARAM_NAME_SW_DECODER = L"swdecoder";
-
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_HQ_SCALER = L"HQSCALER";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_PIP = L"PIP";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_PIP_ZOOM_FACTOR = L"PIPZOOMFACTOR";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_PIP_FOCUS_X = L"PIPFOCUSX";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_PIP_FOCUS_Y = L"PIPFOCUSY";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_SIDE_BY_SIDE = L"SIDEBYSIDE";
 
 PlaybackPipelineBase::PlaybackPipelineBase() :
     m_iVideoWidth(0),
@@ -58,7 +63,8 @@ PlaybackPipelineBase::PlaybackPipelineBase() :
     m_nFfmpegRefCount(0),
     m_bURL(false),
     m_eDecoderFormat(amf::AMF_SURFACE_NV12),
-    m_bCPUDecoder(false)
+    m_bCPUDecoder(false),
+    m_bEnableSideBySide(false)
 {
     g_AMFFactory.Init();
     SetParamDescription(PARAM_NAME_INPUT, ParamCommon,  L"Input file name", NULL);
@@ -72,10 +78,20 @@ PlaybackPipelineBase::PlaybackPipelineBase() :
     SetParamDescription(PARAM_NAME_LOWLATENCY, ParamCommon, L"Low latency mode, boolean, default = false", ParamConverterBoolean);
     SetParamDescription(PARAM_NAME_FULLSCREEN, ParamCommon, L"Specifies fullscreen mode, true, false, default false", ParamConverterBoolean);
     SetParamDescription(PARAM_NAME_SW_DECODER, ParamCommon, L"Forces sw decoder, true, false, default false", ParamConverterBoolean);
-    
+    SetParamDescription(PARAM_NAME_HQ_SCALER, ParamCommon,  L"Use HQ Scaler (OFF, Bilinear, Bicubic, FSR, default = OFF)", ParamConverterHQScalerAlgorithm);
+    SetParamDescription(PARAM_NAME_PIP, ParamCommon, L"Specifies Picture in Picture mode, true, false, default false", ParamConverterBoolean);
+    SetParamDescription(PARAM_NAME_PIP_ZOOM_FACTOR, ParamCommon, L"Specifies magnification ratio of PIP image, int, default 8", ParamConverterDouble);
+    SetParamDescription(PARAM_NAME_PIP_FOCUS_X, ParamCommon, L"H position of foreground image, int, default 0", ParamConverterDouble);
+    SetParamDescription(PARAM_NAME_PIP_FOCUS_Y, ParamCommon, L"V position of foreground image, int, default 0", ParamConverterDouble);
+    SetParamDescription(PARAM_NAME_SIDE_BY_SIDE, ParamCommon, L"Specifies Side-by-Side mode, true, false, default false", ParamConverterBoolean);
 
     SetParam(PARAM_NAME_LISTEN_FOR_CONNECTION, false);
     SetParam(PARAM_NAME_FULLSCREEN, false);
+    SetParam(PARAM_NAME_PIP, false);
+    SetParam(PARAM_NAME_PIP_ZOOM_FACTOR, 0.1f);
+    SetParam(PARAM_NAME_PIP_FOCUS_X, 0.45f);
+    SetParam(PARAM_NAME_PIP_FOCUS_Y, 0.45f);
+    SetParam(PARAM_NAME_SIDE_BY_SIDE, false);
 
 #if defined(_WIN32)
     SetParam(PlaybackPipelineBase::PARAM_NAME_PRESENTER, amf::AMF_MEMORY_DX11);
@@ -136,7 +152,7 @@ AMF_RESULT PlaybackPipelineBase::GetCurrentPts(amf_pts& pts) const
 }
 
 double PlaybackPipelineBase::GetProgressSize() const
-{   
+{
     if (m_pVideoStream != NULL)
     {
         amf_int64 size = 0;
@@ -159,7 +175,7 @@ double PlaybackPipelineBase::GetProgressSize() const
 }
 
 double PlaybackPipelineBase::GetProgressPosition() const
-{   
+{
     if(m_pVideoStream != NULL)
     {
 		amf_int64 pos = 0;
@@ -191,7 +207,7 @@ AMF_RESULT PlaybackPipelineBase::Seek(amf_pts pts)
     {
         amf::AMFMediaSourcePtr pSource(m_pDemuxerVideo);
         if(pSource != NULL)
-        { 
+        {
             Freeze();
             Flush();
             //MM temporarely - till provide better information that pipeline frozen
@@ -200,10 +216,10 @@ AMF_RESULT PlaybackPipelineBase::Seek(amf_pts pts)
 
             if(m_pDemuxerAudio != NULL)
             {
-                
+
                 amf::AMFMediaSourcePtr pSourceAudio(m_pDemuxerAudio);
                 if(pSourceAudio != NULL)
-                { 
+                {
                     pSourceAudio->Seek(pts, amf::AMF_SEEK_PREV_KEYFRAME, -1);
                 }
             }
@@ -279,7 +295,7 @@ AMF_RESULT PlaybackPipelineBase::Init()
     amf::AMFOutputPtr       pAudioOutput;
     amf::AMFOutputPtr       pVideoOutput;
     if( streamType != BitStreamUnknown)
-    { 
+    {
 #if !defined(METRO_APP)
         amf::AMFDataStream::OpenDataStream(inputPath.c_str(), amf::AMFSO_READ, amf::AMFFS_SHARE_READ, &m_pVideoStream);
 #else
@@ -452,7 +468,7 @@ AMF_RESULT PlaybackPipelineBase::Init()
         m_pAudioPresenter->DoActualWait(bTiming);
     }
 
-    
+
     return AMF_OK;
 }
 
@@ -486,6 +502,40 @@ AMF_RESULT PlaybackPipelineBase::InitVideoPipeline(amf_uint32 iVideoStreamIndex,
             Connect(PipelineElementPtr(new AMFComponentElement(m_pVideoProcessor)), m_bCPUDecoder ? 4 : 1, CT_ThreadQueue);
         }
     }
+    if (m_pScaler != NULL)
+    {
+        if (m_pHQScaler2 == NULL)
+        {
+            Connect(PipelineElementPtr(new AMFComponentElement(m_pScaler)), 1, CT_Direct);
+        }
+        else
+        {
+            if (m_pSplitter == NULL)
+            {
+                m_pSplitter = SplitterPtr(new Splitter(true, 2));
+            }
+            if (m_pCombiner == NULL)
+            {
+                m_pCombiner = CombinerPtr(new Combiner(m_pContext));
+            }
+
+            if (m_pSplitter != NULL && m_pCombiner != NULL)
+            {
+                PipelineElementPtr upstreamConnector = GetLastElement();
+
+                PipelineElementPtr hqScalerElement1(new AMFComponentElement(m_pScaler));
+                PipelineElementPtr hqScalerElement2(new AMFComponentElement(m_pHQScaler2));
+
+                Connect(m_pSplitter, 0, upstreamConnector, 0, 4);
+
+                Connect(hqScalerElement1, 0, m_pSplitter, 0, 2);
+                Connect(hqScalerElement2, 0, m_pSplitter, 1, 2);
+
+                Connect(m_pCombiner, 0, hqScalerElement1, 0, 4);
+                Connect(m_pCombiner, 1, hqScalerElement2, 0, 4);
+            }
+        }
+    }
     if(m_bVideoPresenterDirectConnect)
     {
         Connect(m_pVideoPresenter, 4, CT_ThreadPoll); // if back buffers are used in multithreading the video memory changes every Present() call so pointer returned by GetBackBuffer() points to wrong memory
@@ -497,20 +547,52 @@ AMF_RESULT PlaybackPipelineBase::InitVideoPipeline(amf_uint32 iVideoStreamIndex,
 	return AMF_OK;
 }
 
-
-
 AMF_RESULT  PlaybackPipelineBase::InitVideoProcessor()
 {
-    AMF_RESULT res = g_AMFFactory.GetFactory()->CreateComponent(m_pContext, AMFVideoConverter, &m_pVideoProcessor);
-    CHECK_AMF_ERROR_RETURN(res, L"g_AMFFactory.GetFactory()->CreateComponent(" << AMFVideoConverter << L") failed");
+    // check if we need to create the HQ scaler
+    amf_int64  hqScalerMode = -1;
 
-    if (m_pVideoPresenter->SupportAllocator() && m_pContext->GetOpenCLContext() == NULL)
+    AMF_RESULT res = GetParam(PARAM_NAME_HQ_SCALER, hqScalerMode);
+
+    if ((res == AMF_OK) && (hqScalerMode != -1))
     {
-        m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_KEEP_ASPECT_RATIO, true);
-        m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_FILL, true);
-        m_pVideoPresenter->SetProcessor(m_pVideoProcessor);
+        res = g_AMFFactory.GetFactory()->CreateComponent(m_pContext, AMFHQScaler, &m_pScaler);
+        CHECK_AMF_ERROR_RETURN(res, L"g_AMFFactory.GetFactory()->CreateComponent(" << AMFHQScaler << L") failed");
+
+        m_pScaler->SetProperty(AMF_HQ_SCALER_OUTPUT_SIZE, ::AMFConstructSize(m_iVideoWidth, m_iVideoHeight));
+        m_pScaler->SetProperty(AMF_HQ_SCALER_ENGINE_TYPE, m_pVideoPresenter->GetMemoryType());
+        m_pScaler->SetProperty(AMF_HQ_SCALER_ALGORITHM, hqScalerMode);
+
+        /*
+        // If enabled side-by-side, create a separate HQ Scaler
+        if (m_bEnableSideBySide)
+        {
+            res = g_AMFFactory.GetFactory()->CreateComponent(m_pContext, AMFHQScaler, &m_pHQScaler2);
+            CHECK_AMF_ERROR_RETURN(res, L"g_AMFFactory.GetFactory()->CreateComponent(" << AMFHQScaler << L") failed");
+
+            m_pHQScaler2->SetProperty(AMF_HQ_SCALER_OUTPUT_SIZE, ::AMFConstructSize(m_iVideoWidth, m_iVideoHeight));
+            m_pHQScaler2->SetProperty(AMF_HQ_SCALER_ENGINE_TYPE, m_pVideoPresenter->GetMemoryType());
+            m_pHQScaler2->SetProperty(AMF_HQ_SCALER_ALGORITHM, AMF_HQ_SCALER_ALGORITHM_BILINEAR);
+
+            res = m_pHQScaler2->Init(m_pVideoPresenter->GetInputFormat(), m_iVideoWidth, m_iVideoHeight);
+            CHECK_AMF_ERROR_RETURN(res, L"m_pHQScaler2->Init() failed");
+        }
+        */
+
     }
 
+    // initialize scaler
+    if (m_pScaler != nullptr)
+    {
+        res = m_pScaler->Init(m_pVideoPresenter->GetInputFormat(), m_iVideoWidth, m_iVideoHeight);
+        CHECK_AMF_ERROR_RETURN(res, L"m_pScaler->Init() failed with err=%d");
+    }
+
+    // create CSC - this one is needed all the time,
+    // even if we use HQ scaler
+    res = g_AMFFactory.GetFactory()->CreateComponent(m_pContext, AMFVideoConverter, &m_pVideoProcessor);
+    CHECK_AMF_ERROR_RETURN(res, L"g_AMFFactory.GetFactory()->CreateComponent(" << AMFVideoConverter << L") failed");
+
     if (m_eDecoderFormat == amf::AMF_SURFACE_P010) //HDR support
     {
 //        m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_COLOR_PROFILE, AMF_VIDEO_CONVERTER_COLOR_PROFILE_2020);
@@ -524,79 +606,108 @@ AMF_RESULT  PlaybackPipelineBase::InitVideoProcessor()
     m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_OUTPUT_FORMAT, m_pVideoPresenter->GetInputFormat());
 
     res = m_pVideoProcessor->Init(m_eDecoderFormat, m_iVideoWidth, m_iVideoHeight);
+    CHECK_AMF_ERROR_RETURN(res, L"m_pVideoProcessor->Init() failed");
+
+
+    // set-up the processor for presenter - if we use HQ scaler,
+    // we need to set it, otherwise it will be CSC
+
+    if (m_pContext->GetOpenCLContext() == NULL)
+    {
+        m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_KEEP_ASPECT_RATIO, true);
+        m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_FILL, true);
+
+        if (m_pScaler != nullptr)
+        {
+            m_pScaler->SetProperty(AMF_HQ_SCALER_KEEP_ASPECT_RATIO, true);
+            m_pScaler->SetProperty(AMF_HQ_SCALER_FILL, true);
+        }
+
+        m_pVideoPresenter->SetProcessor(m_pVideoProcessor, m_pScaler);
+    }
+
     return res;
 }
 
-AMF_RESULT  PlaybackPipelineBase::InitVideoDecoder(const wchar_t *pDecoderID, amf_int64 codecID, amf_int64 bitrate, AMFRate frameRate, amf::AMFBuffer* pExtraData)
+AMF_RESULT  PlaybackPipelineBase::InitVideoDecoder(const wchar_t *pDecoderID, amf_int64 codecID, amf::AMF_SURFACE_FORMAT surfaceFormat, amf_int64 bitrate, AMFRate frameRate, amf::AMFBuffer* pExtraData)
 {
     bool bLowlatency = false;
     GetParam(PARAM_NAME_LOWLATENCY, bLowlatency);
 
     AMF_VIDEO_DECODER_MODE_ENUM   decoderMode = bLowlatency ? AMF_VIDEO_DECODER_MODE_LOW_LATENCY : AMF_VIDEO_DECODER_MODE_COMPLIANT; // AMF_VIDEO_DECODER_MODE_REGULAR , AMF_VIDEO_DECODER_MODE_LOW_LATENCY;
 
-    AMF_RESULT res = AMF_OK;
-
     bool bSWDecoder = false;
     GetParam(PARAM_NAME_SW_DECODER, bSWDecoder);
-    
+
     if (bSWDecoder == false)
     {
-        res = g_AMFFactory.GetFactory()->CreateComponent(m_pContext, pDecoderID, &m_pVideoDecoder);
+        AMF_RESULT res = g_AMFFactory.GetFactory()->CreateComponent(m_pContext, pDecoderID, &m_pVideoDecoder);
         CHECK_AMF_ERROR_RETURN(res, L"g_AMFFactory.GetFactory()->CreateComponent(" << pDecoderID << L") failed");
     }
-    // check resolution
 
+    // switch to SW Decoder when the HW Decoder does not support this config
     amf::AMFCapsPtr pCaps;
+    if (m_pVideoDecoder != nullptr && m_pVideoDecoder->GetCaps(&pCaps) != AMF_OK)
+    {
+        m_pVideoDecoder = NULL;
+    }
 
     if (m_pVideoDecoder != nullptr)
     {
-        m_pVideoDecoder->GetCaps(&pCaps);
-    }
-    if (pCaps != nullptr)
-    {
-        amf::AMFIOCapsPtr pInputCaps;
-        pCaps->GetInputCaps(&pInputCaps);
-        if (pInputCaps != nullptr)
-        {
-            amf_int32 minWidth = 0;
-            amf_int32 maxWidth = 0;
-            amf_int32 minHeight = 0;
-            amf_int32 maxHeight = 0;
-            pInputCaps->GetWidthRange(&minWidth, &maxWidth);
-            pInputCaps->GetHeightRange(&minHeight, &maxHeight);
-            if (minWidth <= m_iVideoWidth  && m_iVideoWidth <= maxWidth && minHeight <= m_iVideoHeight && m_iVideoHeight <= maxHeight)
-            {
-                m_pVideoDecoder->SetProperty(AMF_VIDEO_DECODER_REORDER_MODE, amf_int64(decoderMode));
-                m_pVideoDecoder->SetProperty(AMF_VIDEO_DECODER_EXTRADATA, amf::AMFVariant(pExtraData));
+        amf::AMFCapsPtr pCaps;
+        AMF_RESULT res = m_pVideoDecoder->GetCaps(&pCaps);
+        CHECK_AMF_ERROR_RETURN(res, L"failed to get decoder caps");
 
-                m_pVideoDecoder->SetProperty(AMF_TIMESTAMP_MODE, amf_int64(AMF_TS_DECODE)); // our sample H264 parser provides decode order timestamps- change depend on demuxer
-                m_eDecoderFormat = amf::AMF_SURFACE_NV12;
-                if (std::wstring(pDecoderID) == AMFVideoDecoderHW_H265_MAIN10)
-                {
-                    m_eDecoderFormat = amf::AMF_SURFACE_P010;
-                }
-                res = m_pVideoDecoder->Init(m_eDecoderFormat, m_iVideoWidth, m_iVideoHeight);
-                CHECK_AMF_ERROR_RETURN(res, L"m_pVideoDecoder->Init(" << m_iVideoWidth << m_iVideoHeight << L") failed " << pDecoderID);
-				if (std::wstring(pDecoderID) == AMFVideoDecoderHW_AV1) 
-				{
-					//Only up to above m_pVideoDecoder->Init(), got/updated actual eDecoderFormat, then also update m_eDecoderFormat here.
-					amf_int64 format = 0;
-					if (m_pVideoDecoder->GetProperty(L"AV1StreamFormat", &format) == AMF_OK)
-					{
-                        m_eDecoderFormat = (amf::AMF_SURFACE_FORMAT)format;
-					}
-				}
-                m_bCPUDecoder = false;
-            }
-            else
+        amf::AMFIOCapsPtr pInputCaps;
+        res = pCaps->GetInputCaps(&pInputCaps);
+        CHECK_AMF_ERROR_RETURN(res, L"failed to get decoder input caps");
+
+        // check resolution
+        amf_int32 minWidth = 0;
+        amf_int32 maxWidth = 0;
+        amf_int32 minHeight = 0;
+        amf_int32 maxHeight = 0;
+        pInputCaps->GetWidthRange(&minWidth, &maxWidth);
+        pInputCaps->GetHeightRange(&minHeight, &maxHeight);
+        if (minWidth <= m_iVideoWidth && m_iVideoWidth <= maxWidth && minHeight <= m_iVideoHeight && m_iVideoHeight <= maxHeight &&
+            surfaceFormat != amf::AMF_SURFACE_P012 && surfaceFormat != amf::AMF_SURFACE_P016) //MM TODO Update caps
+        {
+            m_pVideoDecoder->SetProperty(AMF_VIDEO_DECODER_REORDER_MODE, amf_int64(decoderMode));
+            m_pVideoDecoder->SetProperty(AMF_VIDEO_DECODER_EXTRADATA, amf::AMFVariant(pExtraData));
+
+            m_pVideoDecoder->SetProperty(AMF_TIMESTAMP_MODE, amf_int64(AMF_TS_DECODE)); // our sample H264 parser provides decode order timestamps- change depend on demuxer
+            m_eDecoderFormat = amf::AMF_SURFACE_NV12;
+            if (std::wstring(pDecoderID) == AMFVideoDecoderHW_H265_MAIN10)
             {
-                m_pVideoDecoder = nullptr;
+                m_eDecoderFormat = amf::AMF_SURFACE_P010;
             }
+            res = m_pVideoDecoder->Init(m_eDecoderFormat, m_iVideoWidth, m_iVideoHeight);
+            CHECK_AMF_ERROR_RETURN(res, L"m_pVideoDecoder->Init(" << m_iVideoWidth << L", " << m_iVideoHeight << L") failed " << pDecoderID);
+		    if (std::wstring(pDecoderID) == AMFVideoDecoderHW_AV1)
+		    {
+			    //Only up to above m_pVideoDecoder->Init(), got/updated actual eDecoderFormat, then also update m_eDecoderFormat here.
+			    amf_int64 format = 0;
+			    if (m_pVideoDecoder->GetProperty(L"AV1StreamFormat", &format) == AMF_OK)
+			    {
+                    m_eDecoderFormat = (amf::AMF_SURFACE_FORMAT)format;
+			    }
+		    }
+            m_bCPUDecoder = false;
+        }
+        else
+        {
+            // fall back to software decoder if resolution not supported
+            m_pVideoDecoder = nullptr;
         }
     }
+
+    // use software decoder
+    //   - if specifically requested
+    //   - if resolution not supported
+    //   - if codec not supported
     if (m_pVideoDecoder == nullptr)
     {
-        res = g_AMFFactory.LoadExternalComponent(m_pContext, FFMPEG_DLL_NAME, "AMFCreateComponentInt", (void*)FFMPEG_VIDEO_DECODER, &m_pVideoDecoder);
+        AMF_RESULT res = g_AMFFactory.LoadExternalComponent(m_pContext, FFMPEG_DLL_NAME, "AMFCreateComponentInt", (void*)FFMPEG_VIDEO_DECODER, &m_pVideoDecoder);
         CHECK_AMF_ERROR_RETURN(res, L"LoadExternalComponent(" << FFMPEG_VIDEO_DECODER << L") failed");
         ++m_nFfmpegRefCount;
 
@@ -626,11 +737,37 @@ AMF_RESULT  PlaybackPipelineBase::InitVideoDecoder(const wchar_t *pDecoderID, am
 //      m_pVideoDecoder->SetProperty(AMF_VIDEO_DECODER_REORDER_MODE, amf_int64(decoderMode));
         m_pVideoDecoder->SetProperty(VIDEO_DECODER_EXTRA_DATA, amf::AMFVariant(pExtraData));
 
-        m_eDecoderFormat = amf::AMF_SURFACE_NV12;
+        if(surfaceFormat != amf::AMF_SURFACE_UNKNOWN)
+        {
+            m_eDecoderFormat = surfaceFormat;
+        }
+        else
+        {
+            m_eDecoderFormat = amf::AMF_SURFACE_NV12;
+        }
+/*
         if(std::wstring(pDecoderID) == AMFVideoDecoderHW_H265_MAIN10)
         {
             m_eDecoderFormat = amf::AMF_SURFACE_P010;
         }
+        if (std::wstring(pDecoderID) == AMFVideoDecoderHW_AV1)
+        {
+            // can be 10 or 12 bits per sample
+            amf_int32 outputCount = m_pDemuxerVideo->GetOutputCount();
+            AMF_RETURN_IF_FALSE(outputCount > 0, AMF_FAIL, L"InitVideoDecoder() - not enough output streams");
+
+            amf::AMFOutputPtr pOutput;
+            m_pDemuxerVideo->GetOutput(0, &pOutput);
+
+            amf_int64 surfaceFormat = amf::AMF_SURFACE_UNKNOWN;
+            pOutput->GetProperty(AMF_STREAM_VIDEO_FORMAT, &surfaceFormat);
+
+            if (surfaceFormat != static_cast<amf::AMF_SURFACE_FORMAT>(amf::AMF_SURFACE_UNKNOWN))
+            {
+                m_eDecoderFormat = static_cast<amf::AMF_SURFACE_FORMAT>(surfaceFormat);
+            }
+        }
+*/
         res = m_pVideoDecoder->Init(m_eDecoderFormat, m_iVideoWidth, m_iVideoHeight);
         CHECK_AMF_ERROR_RETURN(res, L"m_pVideoDecoder->Init("<< m_iVideoWidth << m_iVideoHeight << L") failed " << pDecoderID );
         m_bCPUDecoder = true;
@@ -747,13 +884,25 @@ AMF_RESULT PlaybackPipelineBase::Stop()
         m_pVideoDecoder->Terminate();
         m_pVideoDecoder = NULL;
     }
-    
+
     if(m_pVideoProcessor != NULL)
     {
         m_pVideoProcessor->Terminate();
         m_pVideoProcessor = NULL;
     }
 
+    if (m_pScaler != NULL)
+    {
+        m_pScaler->Terminate();
+        m_pScaler = NULL;
+    }
+
+    if (m_pHQScaler2 != NULL)
+    {
+        m_pHQScaler2->Terminate();
+        m_pHQScaler2 = NULL;
+    }
+
     if(m_pVideoPresenter != NULL)
     {
         m_pVideoPresenter->Terminate();
@@ -785,21 +934,81 @@ AMF_RESULT PlaybackPipelineBase::Stop()
 
 void PlaybackPipelineBase::OnParamChanged(const wchar_t* name)
 {
-    if (std::wstring(name) == std::wstring(PARAM_NAME_FULLSCREEN))
+    //check if HQScaler will need to be re-created in the pipeline
+    if (std::wstring(name) == std::wstring(PARAM_NAME_HQ_SCALER))
     {
-        if (m_pVideoPresenter != nullptr)
+        amf_int64  modeHQScaler = -1;
+
+        if (m_pScaler != nullptr)
+        {
+            m_pScaler->GetProperty(AMF_HQ_SCALER_ALGORITHM, &modeHQScaler);
+        }
+
+        amf_int64  modeHQScalerNew = -1;
+        GetParam(PARAM_NAME_HQ_SCALER, modeHQScalerNew);
+
+        if ((modeHQScaler == -1 && modeHQScalerNew != -1)  || 
+            (modeHQScaler != -1 && modeHQScalerNew == -1)) //need to re-build the pipeline
+        {
+            if (GetState() == PipelineStateRunning)
+            {
+                Init();
+                Play();
+            }
+        }
+
+        if (m_pScaler != nullptr && modeHQScalerNew != -1)
+        {
+            m_pScaler->SetProperty(AMF_HQ_SCALER_ALGORITHM, modeHQScalerNew);
+        }
+    }
+
+    if (m_pVideoPresenter != nullptr)
+    {
+        if (std::wstring(name) == std::wstring(PARAM_NAME_FULLSCREEN))
         {
             bool bFullScreen = false;
             GetParam(PARAM_NAME_FULLSCREEN, bFullScreen);
             m_pVideoPresenter->SetFullScreen(bFullScreen);
         }
+        else if (std::wstring(name) == std::wstring(PARAM_NAME_DOTIMING))
+        {
+            bool bDoTiming = true;
+            GetParam(PARAM_NAME_DOTIMING, bDoTiming);
+            m_pVideoPresenter->DoActualWait(bDoTiming);
+        }
+        else if (std::wstring(name) == std::wstring(PARAM_NAME_PIP))
+        {
+            bool bEnablePIP = false;
+            GetParam(PARAM_NAME_PIP, bEnablePIP);
+            m_pVideoPresenter->SetEnablePIP(bEnablePIP);
+        }
+        else if (std::wstring(name) == std::wstring(PARAM_NAME_PIP_ZOOM_FACTOR))
+        {
+            amf_float fPIPZoomFactor;
+            GetParam(PARAM_NAME_PIP_ZOOM_FACTOR, fPIPZoomFactor);
+            m_pVideoPresenter->SetPIPZoomFactor(fPIPZoomFactor);
+        }
+        else if (std::wstring(name) == std::wstring(PARAM_NAME_PIP_FOCUS_X) || std::wstring(name) == std::wstring(PARAM_NAME_PIP_FOCUS_Y))
+        {
+            AMFFloatPoint2D fPIPFocusPos;
+            GetParam(PARAM_NAME_PIP_FOCUS_X, fPIPFocusPos.x);
+            GetParam(PARAM_NAME_PIP_FOCUS_Y, fPIPFocusPos.y);
+
+            m_pVideoPresenter->SetPIPFocusPositions(fPIPFocusPos);
+        }
+        else if (std::wstring(name) == std::wstring(PARAM_NAME_SIDE_BY_SIDE))
+        {
+            bool bEnableSideBySide = false;
+            GetParam(PARAM_NAME_SIDE_BY_SIDE, bEnableSideBySide);
+            m_bEnableSideBySide = bEnableSideBySide;
+        }
     }
     if(m_pVideoProcessor == NULL)
     {
         return;
     }
     UpdateVideoProcessorProperties(name);
-
 }
 
 void PlaybackPipelineBase::UpdateVideoProcessorProperties(const wchar_t* name)
@@ -951,7 +1160,7 @@ AMF_RESULT  PlaybackPipelineBase::InitVideo(amf::AMFOutput* pOutput, amf::AMF_ME
 {
     bool decodeAsAnnexBStream = false; // switches between Annex B and AVCC types of decode input.
 
-    AMF_RESULT res = AMF_OK; 
+    AMF_RESULT res = AMF_OK;
     amf::AMFBufferPtr pExtraData;
     std::wstring pVideoDecoderID;
 
@@ -959,6 +1168,7 @@ AMF_RESULT  PlaybackPipelineBase::InitVideo(amf::AMFOutput* pOutput, amf::AMF_ME
 	double dFps = 0.0;
 
     amf_int64 codecID = 0;
+    amf_int64 surfaceFormat = amf::AMF_SURFACE_NV12;
 
     AMFRate frameRate = {};
 
@@ -977,7 +1187,7 @@ AMF_RESULT  PlaybackPipelineBase::InitVideo(amf::AMFOutput* pOutput, amf::AMF_ME
         }
         m_iVideoWidth = m_pVideoStreamParser->GetPictureWidth();
         m_iVideoHeight = m_pVideoStreamParser->GetPictureHeight();
-    
+
         pVideoDecoderID = m_pVideoStreamParser->GetCodecComponent();
 
 		dFps = m_pVideoStreamParser->GetFrameRate();
@@ -985,7 +1195,7 @@ AMF_RESULT  PlaybackPipelineBase::InitVideo(amf::AMFOutput* pOutput, amf::AMF_ME
         frameRate.den = 1000;
     }
     else if(pOutput != NULL)
-    { 
+    {
         amf::AMFInterfacePtr pInterface;
         pOutput->GetProperty(AMF_STREAM_EXTRA_DATA, &pInterface);
         pExtraData = amf::AMFBufferPtr(pInterface);
@@ -994,25 +1204,27 @@ AMF_RESULT  PlaybackPipelineBase::InitVideo(amf::AMFOutput* pOutput, amf::AMF_ME
         pOutput->GetProperty(AMF_STREAM_VIDEO_FRAME_SIZE, &frameSize);
         m_iVideoWidth = frameSize.width;
         m_iVideoHeight = frameSize.height;
-        
+
         res= pOutput->GetProperty(AMF_STREAM_CODEC_ID, &codecID);
         pVideoDecoderID = StreamCodecIDtoDecoderID(AMF_STREAM_CODEC_ID_ENUM(codecID));
-        
+
         pOutput->SetProperty(AMF_STREAM_ENABLED, true);
 
 		pOutput->GetProperty(AMF_STREAM_BIT_RATE, &bitRate);
 
 		pOutput->GetProperty(AMF_STREAM_VIDEO_FRAME_RATE, &frameRate);
 
+        pOutput->GetProperty(AMF_STREAM_VIDEO_FORMAT, &surfaceFormat);
+
 		dFps = frameRate.den == 0 ? 0.0 : (static_cast<double>(frameRate.num) / static_cast<double>(frameRate.den));
     }
     //---------------------------------------------------------------------------------------------
     // Init Video Decoder
-    res = InitVideoDecoder(pVideoDecoderID.c_str(), codecID, bitRate, frameRate,pExtraData);
+    res = InitVideoDecoder(pVideoDecoderID.c_str(), codecID, (amf::AMF_SURFACE_FORMAT)surfaceFormat, bitRate, frameRate, pExtraData);
     CHECK_AMF_ERROR_RETURN(res, L"InitVideoDecoder() failed");
 
     if(m_pVideoDecoder != NULL)
-    { 
+    {
         m_pVideoDecoder->SetProperty(AMF_TIMESTAMP_MODE, amf_int64(m_pVideoStreamParser != NULL ? AMF_TS_DECODE : AMF_TS_SORT)); // our sample H264 parser provides decode order timestamps- change depend on demuxer
     }
     //---------------------------------------------------------------------------------------------
@@ -1024,7 +1236,15 @@ AMF_RESULT  PlaybackPipelineBase::InitVideo(amf::AMFOutput* pOutput, amf::AMF_ME
     GetParam(PARAM_NAME_FULLSCREEN, bFullScreen);
     m_pVideoPresenter->SetFullScreen(bFullScreen);
 
-    if(m_eDecoderFormat == amf::AMF_SURFACE_P010) //HDR support
+    bool bEnablePIP = false;
+    GetParam(PARAM_NAME_PIP, bEnablePIP);
+    m_pVideoPresenter->SetEnablePIP(bEnablePIP);
+
+    GetParam(PARAM_NAME_SIDE_BY_SIDE, m_bEnableSideBySide);
+
+    if (m_eDecoderFormat == amf::AMF_SURFACE_P010 ||
+        m_eDecoderFormat == amf::AMF_SURFACE_P012 ||
+        m_eDecoderFormat == amf::AMF_SURFACE_P016) //HDR support
     {
         m_pVideoPresenter->SetInputFormat(amf::AMF_SURFACE_RGBA_F16);
 // can be used if needed
@@ -1046,7 +1266,7 @@ void PlaybackPipelineBase::OnEof()
 {
     Pipeline::OnEof();
     if(GetState() == PipelineStateEof)
-    { 
+    {
         bool bLoop = true;
         GetParam(PARAM_NAME_LOOP, bLoop);
         if(bLoop)
@@ -1065,18 +1285,24 @@ void PlaybackPipelineBase::OnEof()
             }
             if(m_pVideoDecoder != NULL)
             {
-
                 m_pVideoDecoder->ReInit(m_iVideoWidth, m_iVideoHeight);
             }
             if(m_pVideoProcessor != NULL)
             {
                 m_pVideoProcessor->ReInit(m_iVideoWidth, m_iVideoHeight);
             }
+            if (m_pScaler != NULL)
+            {
+                m_pScaler->ReInit(m_iVideoWidth, m_iVideoHeight);
+            }
+            if (m_pHQScaler2 != NULL)
+            {
+                m_pHQScaler2->ReInit(m_iVideoWidth, m_iVideoHeight);
+            }
             if(m_pVideoPresenter != NULL)
             {
                 m_pVideoPresenter->Reset();
             }
-
             if(m_pAudioDecoder != NULL)
             {
                 m_pAudioDecoder->ReInit(0, 0);
@@ -1101,10 +1327,19 @@ AMF_RESULT PlaybackPipelineBase::OnActivate(bool bActivated)
             AMFTraceDebug(AMF_FACILITY, L"ProcessMessage() Window Activated");
             m_pVideoPresenter->SetFullScreen(bFullScreen);
 
+            bool bEnablePIP = false;
+            GetParam(PARAM_NAME_PIP, bEnablePIP);
+            m_pVideoPresenter->SetEnablePIP(bEnablePIP);
+
+            AMFFloatPoint2D fPIPFocusPos;
+            GetParam(PARAM_NAME_PIP_FOCUS_X, fPIPFocusPos.x);
+            GetParam(PARAM_NAME_PIP_FOCUS_Y, fPIPFocusPos.y);
+            m_pVideoPresenter->SetPIPFocusPositions(fPIPFocusPos);
         }
         else
         {
             m_pVideoPresenter->SetFullScreen(false);
+            m_pVideoPresenter->SetEnablePIP(false);
             AMFTraceDebug(AMF_FACILITY, L"ProcessMessage() Window Deactivated");
         }
     }
