@@ -91,6 +91,8 @@ AMFAudioEncoderFFMPEGImpl::AMFAudioEncoderFFMPEGImpl(AMFContext* pContext)
     m_PrevPts(-1LL)
 
 {
+    g_AMFFactory.Init();
+
     AMFPrimitivePropertyInfoMapBegin
         AMFPropertyInfoBool(AUDIO_ENCODER_ENABLE_DEBUGGING, L"Enable Debug", false, false),
         AMFPropertyInfoBool(AUDIO_ENCODER_ENABLE_ENCODING, L"Enable encoding", true, false),
@@ -118,6 +120,7 @@ AMFAudioEncoderFFMPEGImpl::AMFAudioEncoderFFMPEGImpl(AMFContext* pContext)
 AMFAudioEncoderFFMPEGImpl::~AMFAudioEncoderFFMPEGImpl()
 {
     Terminate();
+    g_AMFFactory.Terminate();
 }
 //-------------------------------------------------------------------------------------------------
 AMF_RESULT AMF_STD_CALL  AMFAudioEncoderFFMPEGImpl::Init(AMF_SURFACE_FORMAT /*format*/, amf_int32 /*width*/, amf_int32 /*height*/)
@@ -329,10 +332,11 @@ AMF_RESULT AMF_STD_CALL  AMFAudioEncoderFFMPEGImpl::SubmitInput(AMFData* pData)
         return AMF_EOF;
     }
 
-    // if we requested to drain, we shouldn't accept any more input
+    // flush if submitting input to draining encoder
     if (m_bEof)
     {
-        return AMF_EOF;
+        m_bEof = false;
+        Flush();
     }
 
     // update the first frame pts offset
@@ -503,7 +507,7 @@ AMF_RESULT AMF_STD_CALL  AMFAudioEncoderFFMPEGImpl::QueryOutput(AMFData** ppData
         pckt = 1;
         ret = avcodec_encode_audio2(m_pCodecContext, &avPacket, NULL, &pckt);
         if (!pckt)
-        { 
+        {
             m_bDrained = true;
         }
         else
